@@ -32,13 +32,12 @@
 #include "gconf-listeners.h"
 #include "gconf-locale.h"
 #include "gconf-schema.h"
-#include "gconf-glib-private.h"
 #include "gconf.h"
 #include "gconfd.h"
 #include "gconf-database.h"
-#include <orb/orbit.h>
+#include <orbit/orbit.h>
 
-#include "GConf.h"
+#include "GConfX.h"
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -52,6 +51,8 @@
 #include <ctype.h>
 #include <syslog.h>
 #include <time.h>
+#include <sys/wait.h>
+#include <locale.h>
 
 /* This makes hash table safer when debugging */
 #ifndef GCONF_ENABLE_DEBUG
@@ -102,12 +103,14 @@ static void                 shutdown_databases (void);
 static void                 set_default_database (GConfDatabase* db);
 static void                 register_database (GConfDatabase* db);
 static void                 unregister_database (GConfDatabase* db);
-static GConfDatabase*       lookup_database (const gchar *address);
-static GConfDatabase*       obtain_database (const gchar *address,
+static GConfDatabase*       lookup_database (GSList *addresses);
+static GConfDatabase*       obtain_database (GSList *addresses,
                                              GError **err);
 static void                 drop_old_databases (void);
 static gboolean             no_databases_in_use (void);
 
+static void gconf_handle_segv (int signum);
+
 /*
  * Flag indicating that we are shutting down, so return errors
  * on any attempted operation. We do this instead of unregistering with
@@ -117,11 +120,17 @@ static gboolean             no_databases_in_use (void);
 
 static gboolean in_shutdown = FALSE;
 
+/*
+ * Flag indicating we received a SIGHUP and we should reaload
+ * all sources during the next periodic_cleanup()
+ */
+static gboolean need_db_reload = FALSE;
+
 /* 
  * CORBA goo
  */
 
-static ConfigServer server = CORBA_OBJECT_NIL;
+static ConfigServer2 server = CORBA_OBJECT_NIL;
 static PortableServer_POA the_poa;
 static GConfLock *daemon_lock = NULL;
 
@@ -134,6 +143,11 @@ gconfd_get_database(PortableServer_Servant servant,
                     const CORBA_char* address,
                     CORBA_Environment* ev);
 
+static ConfigDatabase
+gconfd_get_database_for_addresses (PortableServer_Servant           servant,
+				   const ConfigServer2_AddressList *addresses,
+				   CORBA_Environment               *ev);
+
 static void
 gconfd_add_client (PortableServer_Servant servant,
                    const ConfigListener client,
@@ -166,8 +180,13 @@ static POA_ConfigServer__epv server_epv = {
   gconfd_shutdown
 };
 
-static POA_ConfigServer__vepv poa_server_vepv = { &base_epv, &server_epv };
-static POA_ConfigServer poa_server_servant = { NULL, &poa_server_vepv };
+static POA_ConfigServer2__epv server2_epv = { 
+  NULL,
+  gconfd_get_database_for_addresses
+};
+
+static POA_ConfigServer2__vepv poa_server_vepv = { &base_epv, &server_epv, &server2_epv };
+static POA_ConfigServer2 poa_server_servant = { NULL, &poa_server_vepv };
 
 static ConfigDatabase
 gconfd_get_default_database(PortableServer_Servant servant,
@@ -192,19 +211,51 @@ gconfd_get_database(PortableServer_Servant servant,
                     CORBA_Environment* ev)
 {
   GConfDatabase *db;
+  GSList *addresses;
   GError* error = NULL;  
 
   if (gconfd_check_in_shutdown (ev))
     return CORBA_OBJECT_NIL;
   
-  db = obtain_database (address, &error);
+  addresses = g_slist_append (NULL, (char *) address);
+  db = obtain_database (addresses, &error);
+  g_slist_free (addresses);
 
   if (db != NULL)
     return CORBA_Object_duplicate (db->objref, ev);
-  else if (gconf_set_exception(&error, ev))
-    return CORBA_OBJECT_NIL;
-  else
+
+  gconf_set_exception (&error, ev);
+
+  return CORBA_OBJECT_NIL;
+}
+
+static ConfigDatabase
+gconfd_get_database_for_addresses (PortableServer_Servant           servant,
+				   const ConfigServer2_AddressList *seq,
+				   CORBA_Environment               *ev)
+{
+  GConfDatabase  *db;
+  GSList         *addresses = NULL;
+  GError         *error = NULL;  
+  int             i;
+
+  if (gconfd_check_in_shutdown (ev))
     return CORBA_OBJECT_NIL;
+
+  i = 0;
+  while (i < seq->_length)
+    addresses = g_slist_append (addresses, seq->_buffer [i++]);
+
+  db = obtain_database (addresses, &error);
+
+  g_slist_free (addresses);
+
+  if (db != NULL)
+    return CORBA_Object_duplicate (db->objref, ev);
+
+  gconf_set_exception (&error, ev);
+
+  return CORBA_OBJECT_NIL;
 }
 
 static void
@@ -258,7 +309,7 @@ gconfd_shutdown(PortableServer_Servant servant, CORBA_Environment *ev)
 static void
 gconf_server_load_sources(void)
 {
-  gchar** addresses;
+  GSList* addresses;
   GList* tmp;
   gboolean have_writable = FALSE;
   gchar* conffile;
@@ -288,13 +339,9 @@ gconf_server_load_sources(void)
   if (addresses == NULL)
     {      
       /* Try using the default address xml:readwrite:$(HOME)/.gconf */
-      addresses = g_new0(gchar*, 2);
+      addresses = g_slist_append(addresses, g_strconcat("xml:readwrite:", g_get_home_dir(), "/.gconf", NULL));
 
-      addresses[0] = g_strconcat("xml:readwrite:", g_get_home_dir(), "/.gconf", NULL);
-
-      addresses[1] = NULL;
-      
-      gconf_log(GCL_DEBUG, _("No configuration files found, trying to use the default config source `%s'"), addresses[0]);
+      gconf_log(GCL_DEBUG, _("No configuration files found, trying to use the default config source `%s'"), (char *)addresses->data);
     }
   
   if (addresses == NULL)
@@ -302,18 +349,16 @@ gconf_server_load_sources(void)
       /* We want to stay alive but do nothing, because otherwise every
          request would result in another failed gconfd being spawned.  
       */
-      const gchar* empty_addr[] = { NULL };
-      gconf_log(GCL_ERR, _("No configuration sources in the source path, configuration won't be saved; edit "GCONF_CONFDIR"/path"));
+      gconf_log(GCL_ERR, _("No configuration sources in the source path, configuration won't be saved; edit %s"), GCONF_CONFDIR"/path");
       /* don't request error since there aren't any addresses */
-      sources = gconf_sources_new_from_addresses(empty_addr, NULL);
+      sources = gconf_sources_new_from_addresses(NULL, NULL);
 
       /* Install the sources as the default database */
       set_default_database (gconf_database_new(sources));
     }
   else
     {
-      sources = gconf_sources_new_from_addresses((const gchar**)addresses,
-                                                 &error);
+      sources = gconf_sources_new_from_addresses(addresses, &error);
 
       if (error != NULL)
         {
@@ -324,7 +369,7 @@ gconf_server_load_sources(void)
           error = NULL;
         }
       
-      g_free(addresses);
+      gconf_address_list_free(addresses);
 
       g_assert(sources != NULL);
 
@@ -365,15 +410,17 @@ signal_handler (int signo)
   
   ++in_fatal;
   
-  switch(signo) {
+  switch (signo) {
     /* Fast cleanup only */
   case SIGSEGV:
   case SIGBUS:
   case SIGILL:
     enter_shutdown ();
-    gconf_log(GCL_ERR,
-              _("Received signal %d, dumping core. Please report a GConf bug."),
-              signo);
+    gconf_log (GCL_ERR,
+               _("Received signal %d, dumping core. Please report a GConf bug."),
+               signo);
+    if (g_getenv ("DISPLAY"))
+      gconf_handle_segv (signo);
     abort ();
     break;
 
@@ -398,7 +445,6 @@ signal_handler (int signo)
     break;
 
   case SIGTERM:
-  case SIGHUP:
     enter_shutdown ();
 
     /* let the fatal signals interrupt us */
@@ -411,7 +457,16 @@ signal_handler (int signo)
       gconf_main_quit ();
     break;
 
+  case SIGHUP:
+    --in_fatal;
+
+    /* reload sources during next periodic_cleanup() */
+    need_db_reload = TRUE;
+    break;
+
   case SIGUSR1:
+    --in_fatal;
+    
     /* it'd be nice to log a message here but it's not very safe, so */
     gconf_log_debug_messages = !gconf_log_debug_messages;
     break;
@@ -422,7 +477,7 @@ signal_handler (int signo)
 }
 
 PortableServer_POA
-gconf_get_poa ()
+gconf_get_poa (void)
 {
   return the_poa;
 }
@@ -435,7 +490,7 @@ log_handler (const gchar   *log_domain,
 {
   GConfLogPriority pri = GCL_WARNING;
   
-  switch (log_level)
+  switch (log_level & G_LOG_LEVEL_MASK)
     {
     case G_LOG_LEVEL_ERROR:
     case G_LOG_LEVEL_CRITICAL:
@@ -462,18 +517,61 @@ log_handler (const gchar   *log_domain,
   gconf_log (pri, "%s", message);
 }
 
+/* From ORBit2 */
+/* There is a DOS attack if another user creates
+ * the given directory and keeps us from creating
+ * it
+ */
+static gboolean
+test_safe_tmp_dir (const char *dirname)
+{
+  struct stat statbuf;
+  int fd;
+
+  fd = open (dirname, O_RDONLY);  
+  if (fd < 0)
+    {
+      gconf_log (GCL_WARNING, _("Failed to open %s: %s"),
+                 dirname, g_strerror (errno));
+      return FALSE;
+    }
+  
+  if (fstat (fd, &statbuf) != 0)
+    {
+      gconf_log (GCL_WARNING, _("Failed to stat %s: %s"),
+                 dirname, g_strerror (errno));
+      close (fd);
+      return FALSE;
+    }
+  close (fd);
+  
+  if (statbuf.st_uid != getuid ())
+    {
+      gconf_log (GCL_WARNING, _("Owner of %s is not the current user"),
+                 dirname);
+      return FALSE;
+    }
+  
+  if ((statbuf.st_mode & (S_IRWXG|S_IRWXO)) ||
+      !S_ISDIR (statbuf.st_mode))
+    {
+      gconf_log (GCL_WARNING, _("Bad permissions %lo on directory %s"),
+                 (unsigned long) statbuf.st_mode & 07777, dirname);
+      return FALSE;
+    }
+  
+  return TRUE;
+}
+
 int 
 main(int argc, char** argv)
 {
   struct sigaction act;
   sigset_t empty_mask;
-  /*   PortableServer_ObjectId objid = {0, sizeof("ConfigServer"), "ConfigServer"}; */
-  PortableServer_ObjectId* objid;
   CORBA_Environment ev;
   CORBA_ORB orb;
   gchar* logname;
   const gchar* username;
-  guint len;
   gchar* ior;
   int exit_code = 0;
   GError *err;
@@ -481,6 +579,10 @@ main(int argc, char** argv)
   char *gconfd_dir;
   int dev_null_fd;
   int write_byte_fd;
+
+  _gconf_init_i18n ();
+  setlocale (LC_ALL, "");
+  textdomain (GETTEXT_PACKAGE);
   
   /* Now this is an argument parser */
   if (argc > 1)
@@ -488,11 +590,15 @@ main(int argc, char** argv)
   else
     write_byte_fd = -1;
   
-  chdir ("/");
-
   /* This is so we don't prevent unmounting of devices. We divert
    * all messages to syslog
    */
+  if (chdir ("/") < 0)
+    {
+       g_printerr ("Could not change to root directory: %s\n",
+		g_strerror (errno));
+       exit (1);
+    }
 
   if (!g_getenv ("GCONF_DEBUG_OUTPUT"))
     {
@@ -504,6 +610,10 @@ main(int argc, char** argv)
 	  dup2 (dev_null_fd, 2);
 	}
     }
+  else
+    {
+      gconf_log_debug_messages = TRUE;
+    }
   
   umask (022);
   
@@ -511,14 +621,18 @@ main(int argc, char** argv)
   
   /* Logs */
   username = g_get_user_name();
-  len = strlen(username) + strlen("gconfd") + 15;
-  logname = g_malloc(len);
-  g_snprintf(logname, len, "gconfd (%s-%u)", username, (guint)getpid());
+  logname = g_strdup_printf("gconfd (%s-%u)", username, (guint)getpid());
 
   openlog (logname, LOG_NDELAY, LOG_USER);
 
   g_log_set_handler (NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
                      log_handler, NULL);
+
+  g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
+                     log_handler, NULL);
+
+  g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
+                     log_handler, NULL);
   
   /* openlog() does not copy logname - what total brokenness.
      So we free it at the end of main() */
@@ -535,30 +649,29 @@ main(int argc, char** argv)
   act.sa_handler = signal_handler;
   act.sa_mask    = empty_mask;
   act.sa_flags   = 0;
-  sigaction (SIGTERM,  &act, 0);
-  sigaction (SIGILL,  &act, 0);
-  sigaction (SIGBUS,  &act, 0);
-  sigaction (SIGFPE,  &act, 0);
-  sigaction (SIGHUP,  &act, 0);
-  sigaction (SIGSEGV, &act, 0);
-  sigaction (SIGABRT, &act, 0);
-
+  sigaction (SIGTERM,  &act, NULL);
+  sigaction (SIGILL,  &act, NULL);
+  sigaction (SIGBUS,  &act, NULL);
+  sigaction (SIGFPE,  &act, NULL);
+  sigaction (SIGHUP,  &act, NULL);
+  sigaction (SIGSEGV, &act, NULL);
+  sigaction (SIGABRT, &act, NULL);
+  sigaction (SIGUSR1,  &act, NULL);
+  
   act.sa_handler = SIG_IGN;
-  sigaction (SIGINT, &act, 0);
+  sigaction (SIGINT, &act, NULL);
 
   CORBA_exception_init(&ev);
 
   init_databases ();
 
-  orb = gconf_orb_get();
+  orb = gconf_orb_get ();
   
-  POA_ConfigServer__init(&poa_server_servant, &ev);
+  POA_ConfigServer2__init (&poa_server_servant, &ev);
   
   the_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
   PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(the_poa, &ev), &ev);
 
-  objid = PortableServer_POA_activate_object(the_poa, &poa_server_servant, &ev);
-  
   server = PortableServer_POA_servant_to_reference(the_poa,
                                                    &poa_server_servant,
                                                    &ev);
@@ -575,14 +688,28 @@ main(int argc, char** argv)
 
   gconfd_dir = gconf_get_daemon_dir ();
   lock_dir = gconf_get_lock_dir ();
-
+  
   if (mkdir (gconfd_dir, 0700) < 0 && errno != EEXIST)
     gconf_log (GCL_WARNING, _("Failed to create %s: %s"),
                gconfd_dir, g_strerror (errno));
-  
-  
-  err = NULL;
-  daemon_lock = gconf_get_lock (lock_dir, &err);
+
+  if (!test_safe_tmp_dir (gconfd_dir))
+    {
+      err = g_error_new (GCONF_ERROR,
+                         GCONF_ERROR_LOCK_FAILED,
+                         _("Directory %s has a problem, gconfd can't use it"),
+                         gconfd_dir);
+      daemon_lock = NULL;
+    }
+  else
+    {
+      err = NULL;
+      
+      daemon_lock = gconf_get_lock (lock_dir, &err);
+    }
+
+  g_free (gconfd_dir);
+  g_free (lock_dir);
 
   if (daemon_lock != NULL)
     {
@@ -602,7 +729,7 @@ main(int argc, char** argv)
       char buf[1] = { 'g' };
       if (write (write_byte_fd, buf, 1) != 1)
         {
-          gconf_log (GCL_ERR, _("Failed to write byte to pipe fd %d so client program may hang: %s"), write_byte_fd, g_strerror (errno));
+          gconf_log (GCL_ERR, _("Failed to write byte to pipe file descriptor %d so client program may hang: %s"), write_byte_fd, g_strerror (errno));
         }
 
       close (write_byte_fd);
@@ -620,7 +747,7 @@ main(int argc, char** argv)
       shutdown_databases ();
       
       return 1;
-    }
+    }  
 
   /* Read saved log file, if any */
   logfile_read ();
@@ -644,10 +771,6 @@ main(int argc, char** argv)
 
   gconfd_locale_cache_drop ();
 
-  /* Now we can release the lock */
-
-  server = CORBA_OBJECT_NIL;
-
   if (daemon_lock)
     {
       err = NULL;
@@ -663,8 +786,9 @@ main(int argc, char** argv)
   daemon_lock = NULL;
   
   gconf_log (GCL_INFO, _("Exiting"));
-  
-  g_free (logname);
+
+  /* Can't do this due to stupid atexit() handler that calls g_log stuff */
+  /*   g_free (logname); */
   
   return exit_code;
 }
@@ -680,6 +804,18 @@ static gboolean need_log_cleanup = FALSE;
 static gboolean
 periodic_cleanup_timeout(gpointer data)
 {  
+  if (need_db_reload)
+    {
+      gconf_log (GCL_INFO, _("SIGHUP received, reloading all databases"));
+
+      need_db_reload = FALSE;
+      logfile_save ();
+      shutdown_databases ();
+      init_databases ();
+      gconf_server_load_sources ();
+      logfile_read ();
+    }
+  
   gconf_log (GCL_DEBUG, "Performing periodic cleanup, expiring cache cruft");
   
   drop_old_clients ();
@@ -720,15 +856,11 @@ gconf_main(void)
 {
   GMainLoop* loop;
 
-  loop = g_main_new(TRUE);
+  loop = g_main_loop_new (NULL, TRUE);
 
   if (main_loops == NULL)
     {
-#ifdef GCONF_ENABLE_DEBUG
-      gulong timeout_len = 1000*60*1; /* 1 sec * 60 s/min * 1 min */
-#else
-      gulong timeout_len = 1000*60*15; /* 1 sec * 60 s/min * 2 min */
-#endif
+      gulong timeout_len = 1000*60*0.5; /* 1 sec * 60 s/min * .5 min */
       
       g_assert(timeout_id == 0);
       timeout_id = g_timeout_add (timeout_len,
@@ -739,7 +871,7 @@ gconf_main(void)
   
   main_loops = g_slist_prepend(main_loops, loop);
 
-  g_main_run(loop);
+  g_main_loop_run (loop);
 
   main_loops = g_slist_remove(main_loops, loop);
 
@@ -750,7 +882,7 @@ gconf_main(void)
       timeout_id = 0;
     }
   
-  g_main_destroy(loop);
+  g_main_loop_unref (loop);
 }
 
 static void 
@@ -758,7 +890,7 @@ gconf_main_quit(void)
 {
   g_return_if_fail(main_loops != NULL);
 
-  g_main_quit(main_loops->data);
+  g_main_loop_quit (main_loops->data);
 }
 
 static gboolean
@@ -772,7 +904,7 @@ gconf_main_is_running (void)
  */
 
 static GList* db_list = NULL;
-static GHashTable* dbs_by_address = NULL;
+static GHashTable* dbs_by_addresses = NULL;
 static GConfDatabase *default_db = NULL;
 
 static void
@@ -781,13 +913,9 @@ init_databases (void)
   gconfd_need_log_cleanup ();
   
   g_assert(db_list == NULL);
-  g_assert(dbs_by_address == NULL);
+  g_assert(dbs_by_addresses == NULL);
   
-  dbs_by_address = g_hash_table_new (g_str_hash, g_str_equal);
-
-  /* Default database isn't in the address hash since it has
-     multiple addresses in a stack
-  */
+  dbs_by_addresses = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
@@ -796,10 +924,8 @@ set_default_database (GConfDatabase* db)
   gconfd_need_log_cleanup ();
   
   default_db = db;
-  
-  /* Default database isn't in the address hash since it has
-     multiple addresses in a stack
-  */
+
+  register_database (db);
 }
 
 static void
@@ -808,9 +934,9 @@ register_database (GConfDatabase *db)
   gconfd_need_log_cleanup ();
   
   if (db->sources->sources)
-    safe_g_hash_table_insert(dbs_by_address,
-                             ((GConfSource*)db->sources->sources->data)->address,
-                             db);
+    safe_g_hash_table_insert (dbs_by_addresses,
+			      (char *) gconf_database_get_persistent_name (db),
+			      db);
   
   db_list = g_list_prepend (db_list, db);
 }
@@ -821,8 +947,10 @@ unregister_database (GConfDatabase *db)
   gconfd_need_log_cleanup ();
   
   if (db->sources->sources)
-    g_hash_table_remove(dbs_by_address,
-                        ((GConfSource*)(db->sources->sources->data))->address);
+    {
+      g_hash_table_remove (dbs_by_addresses,
+			   gconf_database_get_persistent_name (db));
+    }
 
   db_list = g_list_remove (db_list, db);
 
@@ -830,30 +958,36 @@ unregister_database (GConfDatabase *db)
 }
 
 static GConfDatabase*
-lookup_database (const gchar *address)
+lookup_database (GSList *addresses)
 {
-  if (address == NULL)
+  GConfDatabase *retval;
+  char          *key;
+
+  if (addresses == NULL)
     return default_db;
-  else
-    return g_hash_table_lookup (dbs_by_address, address);
+
+  key = gconf_address_list_get_persistent_name (addresses);
+
+  retval = g_hash_table_lookup (dbs_by_addresses, key);
+
+  g_free (key);
+
+  return retval;
 }
 
 static GConfDatabase*
-obtain_database (const gchar *address,
+obtain_database (GSList  *addresses,
                  GError **err)
 {
-  
   GConfSources* sources;
-  const gchar* addresses[] = { NULL, NULL };
   GError* error = NULL;
   GConfDatabase *db;
 
-  addresses[0] = address;
-  db = lookup_database (address);
+  db = lookup_database (addresses);
 
   if (db)
     return db;
-  
+
   sources = gconf_sources_new_from_addresses(addresses, &error);
 
   if (error != NULL)
@@ -892,6 +1026,12 @@ drop_old_databases(void)
     {
       GConfDatabase* db = tmp_list->data;
 
+      if (db == default_db)
+	{
+	  tmp_list = g_list_next (tmp_list);
+	  continue;
+	}
+
       /* Drop any listeners whose clients are gone. */
       gconf_database_drop_dead_listeners (db);
       
@@ -941,14 +1081,10 @@ shutdown_databases (void)
   g_list_free (db_list);
   db_list = NULL;
 
-  if (dbs_by_address)
-    g_hash_table_destroy(dbs_by_address);
-
-  dbs_by_address = NULL;
-
-  if (default_db)
-    gconf_database_free (default_db);
+  if (dbs_by_addresses)
+    g_hash_table_destroy(dbs_by_addresses);
 
+  dbs_by_addresses = NULL;
   default_db = NULL;
 }
 
@@ -962,6 +1098,76 @@ no_databases_in_use (void)
     gconf_listeners_count (default_db->listeners) == 0;
 }
 
+void
+gconfd_notify_other_listeners (GConfDatabase *modified_db,
+			       GConfSources  *modified_sources,
+                               const char    *key)
+{
+  GList *tmp;
+
+  if (!modified_sources)
+    return;
+  
+  tmp = db_list;
+  while (tmp != NULL)
+    {
+      GConfDatabase *db = tmp->data;
+
+      if (db != modified_db)
+	{
+	  GList *tmp2;
+
+	  tmp2 = modified_sources->sources;
+	  while (tmp2)
+	    {
+	      GConfSource *modified_source = tmp2->data;
+
+	      if (gconf_sources_is_affected (db->sources, modified_source, key))
+		{
+		  GConfValue  *value;
+		  ConfigValue *cvalue;
+		  GError      *error;
+		  gboolean     is_default;
+		  gboolean     is_writable;
+
+		  error = NULL;
+		  value = gconf_database_query_value (db,
+						      key,
+						      NULL,
+						      TRUE,
+						      NULL,
+						      &is_default,
+						      &is_writable,
+						      &error);
+		  if (error != NULL)
+		    {
+		      gconf_log (GCL_WARNING,
+				 _("Error obtaining new value for `%s': %s"),
+				 key, error->message);
+		      g_error_free (error);
+		      return;
+		    }
+
+		  cvalue = gconf_corba_value_from_gconf_value (value);
+		  gconf_database_notify_listeners (db,
+						   NULL,
+						   key,
+						   cvalue,
+						   is_default,
+						   is_writable,
+						   FALSE);
+		  CORBA_free (cvalue);
+		  gconf_value_free (value);
+		}
+
+	      tmp2 = tmp2->next;
+	    }
+	}
+
+      tmp = tmp->next;
+    }
+}
+
 /*
  * Cleanup
  */
@@ -1056,7 +1262,7 @@ gconf_set_exception(GError** error,
     CORBA_exception_set(ev, CORBA_USER_EXCEPTION,
                         ex_ConfigException, ce);
 
-    gconf_log(GCL_ERR, _("Returning exception: %s"), (*error)->message);
+    gconf_log(GCL_DEBUG, _("Returning exception: %s"), (*error)->message);
       
     g_error_free(*error);
     *error = NULL;
@@ -1227,26 +1433,19 @@ logfile_save (void)
                          * that matter on open()
                          */
 
-  saveme = g_string_new ("");
+  saveme = g_string_new (NULL);
 
   /* Clients */
   log_clients_to_string (saveme);
   
-  /* Default database */
-  gconf_database_log_listeners_to_string (default_db,
-                                          TRUE,
-                                          saveme);
-
-  /* Other databases */
-  
+  /* Databases */
   tmp_list = db_list;
-
   while (tmp_list)
     {
       GConfDatabase *db = tmp_list->data;
 
       gconf_database_log_listeners_to_string (db,
-                                              FALSE,
+                                              db == default_db ? TRUE : FALSE,
                                               saveme);
       
       tmp_list = g_list_next (tmp_list);
@@ -1407,7 +1606,7 @@ parse_listener_entry (GHashTable *entries,
       return FALSE;
     }
   
-  while (*p && isspace (*p))
+  while (*p && g_ascii_isspace (*p))
     ++p;
 
   errno = 0;
@@ -1430,7 +1629,7 @@ parse_listener_entry (GHashTable *entries,
   
   p = end;
 
-  while (*p && isspace (*p))
+  while (*p && g_ascii_isspace (*p))
     ++p;
 
   err = NULL;
@@ -1450,7 +1649,7 @@ parse_listener_entry (GHashTable *entries,
   address = p;
   p = end;
 
-  while (*p && isspace (*p))
+  while (*p && g_ascii_isspace (*p))
     ++p;
   
   err = NULL;
@@ -1470,7 +1669,7 @@ parse_listener_entry (GHashTable *entries,
   location = p;
   p = end;
 
-  while (*p && isspace (*p))
+  while (*p && g_ascii_isspace (*p))
     ++p;
   
   err = NULL;
@@ -1574,7 +1773,7 @@ parse_client_entry (GHashTable *clients,
       return FALSE;
     }
   
-  while (*p && isspace (*p))
+  while (*p && g_ascii_isspace (*p))
     ++p;
   
   err = NULL;
@@ -1640,9 +1839,7 @@ restore_client (const gchar *ior)
   
   CORBA_exception_init (&ev);
   
-  cl = CORBA_ORB_string_to_object (gconf_orb_get (),
-                                   (gchar*)ior,
-                                   &ev);
+  cl = CORBA_ORB_string_to_object (gconf_orb_get (), (gchar*)ior, &ev);
 
   CORBA_exception_free (&ev);
   
@@ -1689,9 +1886,7 @@ restore_listener (GConfDatabase* db,
   
   CORBA_exception_init (&ev);
   
-  cl = CORBA_ORB_string_to_object (gconf_orb_get (),
-                                   lle->ior,
-                                   &ev);
+  cl = CORBA_ORB_string_to_object (gconf_orb_get (), lle->ior, &ev);
 
   CORBA_exception_free (&ev);
   
@@ -1722,11 +1917,11 @@ restore_listener (GConfDatabase* db,
                  "Failed to cancel previous daemon's listener in saved state file: %s",
                  err->message);
       g_error_free (err);
-    }  
+    }
+  
+  new_cnxn = gconf_database_readd_listener (db, cl, "from-saved-state", lle->location);
 
-  new_cnxn = gconf_database_readd_listener (db, cl, lle->location);
-
-  gconf_log (GCL_DEBUG, "Attempting to update listener from saved state file, old connection %u, new connectin %u", (guint) lle->connection_id, (guint) new_cnxn);
+  gconf_log (GCL_DEBUG, "Attempting to update listener from saved state file, old connection %u, new connection %u", (guint) lle->connection_id, (guint) new_cnxn);
   
   ConfigListener_update_listener (cl,
                                   db->objref,
@@ -1781,12 +1976,20 @@ listener_logentry_restore_and_destroy_foreach (gpointer key,
                                                gpointer data)
 {
   ListenerLogEntry *lle = key;
-  GConfDatabase *db;
+  GConfDatabase *db = NULL;
   
   if (strcmp (lle->address, "def") == 0)
     db = default_db;
   else
-    db = obtain_database (lle->address, NULL);
+    {
+      GSList *addresses;
+
+      addresses = gconf_persistent_name_get_address_list (lle->address);
+
+      db = obtain_database (addresses, NULL);
+
+      gconf_address_list_free (addresses);
+    }
   
   if (db == NULL)
     {
@@ -1823,7 +2026,7 @@ read_line (FILE *f)
   int c;
   GString *str;
   
-  str = g_string_new ("");
+  str = g_string_new (NULL);
   
   flockfile (f);
 
@@ -1890,8 +2093,9 @@ logfile_read (void)
   
   if (f == NULL)
     {
-      gconf_log (GCL_ERR, _("Unable to open saved state file '%s': %s"),
-                 logfile, g_strerror (errno));
+      if (errno != ENOENT)
+          gconf_log (GCL_ERR, _("Unable to open saved state file '%s': %s"),
+                     logfile, g_strerror (errno));
 
       goto finished;
     }
@@ -1944,6 +2148,8 @@ logfile_read (void)
   g_slist_free (lines);
   
  finished:
+  if (f != NULL)
+    fclose (f);
   
   g_free (logfile);
   g_free (logdir);
@@ -2105,8 +2311,8 @@ add_client (const ConfigListener client)
   gconfd_need_log_cleanup ();
   
   if (client_table == NULL)
-    client_table = g_hash_table_new ((GHashFunc) g_CORBA_Object_hash,
-                                     (GCompareFunc) g_CORBA_Object_equal);
+    client_table = g_hash_table_new ((GHashFunc) gconf_CORBA_Object_hash,
+                                     (GCompareFunc) gconf_CORBA_Object_equal);
 
   if (g_hash_table_lookup (client_table, client))
     {
@@ -2120,12 +2326,21 @@ add_client (const ConfigListener client)
     {
       CORBA_Environment ev;
       ConfigListener copy;
+      ORBitConnection *connection;
       
       CORBA_exception_init (&ev);
       copy = CORBA_Object_duplicate (client, &ev);
       g_hash_table_insert (client_table, copy, copy);
       CORBA_exception_free (&ev);
 
+      /* Set maximum buffer size, which makes the connection nonblocking
+       * if the kernel buffers are full and keeps gconfd from
+       * locking up. Set the max to a pretty high number to avoid
+       * dropping clients that are just stuck for a while.
+       */
+      connection = ORBit_small_get_connection (copy);
+      ORBit_connection_set_max_buffer (connection, 1024 * 128);
+      
       log_client_add (client);
 
       gconf_log (GCL_DEBUG, "Added a new client");
@@ -2246,22 +2461,31 @@ drop_old_clients (void)
       while (tmp != NULL)
         {
           ConfigListener cl = tmp->data;
-          
-          ConfigListener_ping (cl, &ev);
+          CORBA_boolean result;
 
+          result = CORBA_Object_non_existent (cl, &ev);
+          
           if (ev._major != CORBA_NO_EXCEPTION)
             {
-              gconf_log (GCL_DEBUG, "Removing stale client");
+              gconf_log (GCL_WARNING, "Exception from CORBA_Object_non_existent(), assuming stale listener");
+              CORBA_exception_free (&ev);
+              CORBA_exception_init (&ev);
+              result = TRUE;
+            }
+
+          if (result)
+            {
+              gconf_log (GCL_DEBUG, "removing stale client in drop_old_clients");
               
               remove_client (cl);
-              
-              CORBA_exception_free (&ev);
             }
           
           tmp = g_slist_next (tmp);
         }
 
       g_slist_free (clients);
+
+      CORBA_exception_free (&ev);
     }
 }
 
@@ -2273,3 +2497,11 @@ client_count (void)
   else
     return g_hash_table_size (client_table);
 }
+
+
+static void
+gconf_handle_segv (int signum)
+{
+  return;
+}
+
